use strict;
use Test::More tests => 282;

use DBI;
use Data::Dumper;
use File::Basename qw(dirname);
use File::Spec;
use Getopt::Long;
use Config;

use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::File qw( RealPath GetFileContent RunOnDirectory);
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::SystemCommand qw (SystemCommand);
use DE_EPAGES::Database::API::Connection qw (ConnectSite GetCurrentDBHandle);
use DE_EPAGES::Database::API::OrderFile qw( GetSqlFileOrder );

sub Prepare {
    my ($CurrentDir) = @_;

    my $PerlExe = $^O eq 'MSWin32' ? 'perl' : $ENV{PERL};

    RunOnDirectory( $CurrentDir, sub {
        #drop patch system info db
        SystemCommand([$PerlExe, 'Scripts/dropPatchDB.pl'], {'Print' => 1});

        #generate patch system info db
        SystemCommand([$PerlExe, 'Scripts/createPatchDB.pl'], {'Print' => 1});
    });

    #connect to patch system info db
    ConnectSite('PatchSystemInfoDB');
    my $dbh = GetCurrentDBHandle();
    $dbh->{RaiseError} = 1;

    #get sql files in right order
    my $aFileNames = GetSqlFileOrder("$CurrentDir/Database/mysql");

    #for each filename process sql statements
    foreach my $FileName (@$aFileNames){
        my $rContent = GetFileContent( $FileName );

        #foreach sql statement in file
        foreach my $SQLStatement ( grep { /\S/ } split /\ngo\b/i, $$rContent ) {
            print "$SQLStatement\n\n";
            #execute it
            $dbh->do($SQLStatement);
        }
    }

    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('1','1.1.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('2','1.1.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('3','1.1.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('4','1.2.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('5','1.2.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('6','1.2.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('7','2.1.1')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('8','2.1.2')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('9','2.1.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('10','6.17')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('11','6.17.9')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('12','6.17.9.3')");
    $dbh->do("INSERT INTO patchversion (orderid,label) VALUES ('13','6.17.9.3.1')");
    $dbh->do("INSERT INTO patchsystemattribute (keyname,value) VALUES ('ePagesStartVersion','1.1.1')");
    $dbh->do("INSERT INTO patchsystemattribute (keyname,value) VALUES ('ePagesTargetVersion','2.1.3')");

}

sub Cleanup {
    my ($CurrentDir) = @_;

    my $PerlExe = $^O eq 'MSWin32' ? 'perl' : $ENV{PERL};
    RunOnDirectory( $CurrentDir, sub {
        #drop patch system info db
        SystemCommand([$PerlExe, 'Scripts/dropPatchDB.pl'], {'Print' => 1});
    });
}

sub Main {
    local $| = 1;

    my $Help;
    GetOptions( 'help' => \$Help );
    usage() if $Help;

    my $CurrentDir = dirname(dirname(File::Spec->rel2abs($0)));

    #test preperation
    Prepare($CurrentDir);

    #get patchsysteminfohandler module file name
    my $PatchSystemInfoHandlerModul = "$CurrentDir/API/Patch/PatchSystemInfoHandler.pm";
    my $PatchSystemInfoHandler;

    RunOnDirectory($CurrentDir, sub {
        #use file
        do $PatchSystemInfoHandlerModul; die $@ if $@;

        #test 01 - exists patch system info
        $PatchSystemInfoHandler = API::Patch::PatchSystemInfoHandler->new();

        #initialize PatchSystemInfoDB
        $PatchSystemInfoHandler->initializePatchSystemInfoDB();
    });

    $PatchSystemInfoHandler->setStartTime();
    my $StartTime = $PatchSystemInfoHandler->getStartTime();
    
    #set values
    my $PatchDir = $PatchSystemInfoHandler->getBasicAttribute('PatchDir');
    my $PatchExtractedDataPackDir = $PatchDir . '/ExtractedDataPacks';
    my $PatchExtractedWebRootPackVersionDir = $PatchExtractedDataPackDir . '/ExtractedWebRootPacks.v';
    my $EpagesSharedDir = RealPath($ENV{EPAGES_SHARED});
    my $EpagesCartridgesDir = RealPath($ENV{'EPAGES_CARTRIDGES'});
    my $EpagesSharedPatchesDir = "$EpagesSharedDir/Patches";
    my $EpagesWebRootPackDir = "$EpagesSharedPatchesDir/WebRootPacks";
    my $EpagesStatic = RealPath($ENV{'EPAGES_STATIC'});
    my $EpagesLanguagePackDir = "$EpagesSharedPatchesDir/LanguagePacks";

    $PatchSystemInfoHandler->setBasicAttribute('PatchExtractedWebRootPackVersionDir', $PatchExtractedWebRootPackVersionDir);
    $PatchSystemInfoHandler->setBasicAttribute('EpagesCartridgesDir', $EpagesCartridgesDir);
    $PatchSystemInfoHandler->setBasicAttribute('EpagesSharedPatchesDir', $EpagesSharedPatchesDir);
    $PatchSystemInfoHandler->setBasicAttribute('EpagesCompiledTemplatesPackDir', $EpagesSharedPatchesDir . '/CtmplPacks');
    $PatchSystemInfoHandler->setBasicAttribute('EpagesStatic', $EpagesStatic);
    $PatchSystemInfoHandler->setBasicAttribute('EpagesWebRootPackDir', $EpagesWebRootPackDir);
    $PatchSystemInfoHandler->setBasicAttribute('EpagesLanguagePackDir', $EpagesLanguagePackDir);
    $PatchSystemInfoHandler->setBasicAttribute('PatchExtractedDataPackDir', $PatchExtractedDataPackDir);
    
    #system
    my $OS = $Config{'osname'};

    is ($PatchSystemInfoHandler->getOSName(), $OS , 'OSName is correct');

    #hosts
    ok(not ($PatchSystemInfoHandler->existsAlphaHost()), 'alpha host not exists');

    ok(not ($PatchSystemInfoHandler->existsHost('testhost1')), "'testhost1' not exists");

    $PatchSystemInfoHandler->addHost('testhost1');

    ok($PatchSystemInfoHandler->existsHost('testhost1'), "'testhost1' now exists");

    ok(not ($PatchSystemInfoHandler->existsAlphaHost()), 'alpha host not exists');

    $PatchSystemInfoHandler->addHost('alphatesthost1',1);

    ok($PatchSystemInfoHandler->existsAlphaHost(), 'alpha host now exists');

    eval{
        $PatchSystemInfoHandler->addHost('alphatesthost2',1);
    };
    ok( ExistsError(), 'error trying add second alpha host');
    ok(not ($PatchSystemInfoHandler->existsHost('alphatesthost2')), "'alphatesthost2' not exists");

    ok(('alphatesthost1' eq $PatchSystemInfoHandler->getAlphaHost()), 'get correct alpha host');

    ok($PatchSystemInfoHandler->isAlphaHost('alphatesthost1'), "'alphatesthost1' is alpha host");

    $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchInstallHandler','alphatesthost1');

    my @Hosts = ( 'alphatesthost1' );

    is_deeply ($PatchSystemInfoHandler->getHandlerHosts('API::Patch::PatchInstallHandler'), \@Hosts, 'hosts for handler correct');

    @Hosts = ('alphatesthost1', 'testhost1');

    is_deeply ($PatchSystemInfoHandler->getHosts(), \@Hosts, 'hosts correct');

    my @HostHandlers = ('API::Patch::PatchInstallHandler');

    is_deeply ($PatchSystemInfoHandler->getHostHandlers('alphatesthost1'), \@HostHandlers, 'handlers for host correct');

    @HostHandlers = ();

    is_deeply ($PatchSystemInfoHandler->getHostHandlers('testhost1'), \@HostHandlers, 'handlers for host correct');

    $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchPrepareHandler','alphatesthost1');
    $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchDownloadHandler','testhost1');

    @HostHandlers = ('API::Patch::PatchPrepareHandler', 'API::Patch::PatchInstallHandler');

    is_deeply ($PatchSystemInfoHandler->getHostHandlers('alphatesthost1'), \@HostHandlers, 'handlers for host correct');

    @HostHandlers = ('API::Patch::PatchDownloadHandler');

    is_deeply ($PatchSystemInfoHandler->getHostHandlers('testhost1'), \@HostHandlers, 'handlers for host correct');

    eval{
        $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchInstallHandler','alphatesthost2');
    };
    ok( ExistsError(), 'error trying bind non existing host');

    $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchInstallHandler','testhost1');

    @Hosts = ( 'alphatesthost1', 'testhost1' );

    is_deeply ($PatchSystemInfoHandler->getHandlerHosts('API::Patch::PatchInstallHandler'), \@Hosts, 'hosts for handler correct');


    #attributes
    ok($PatchSystemInfoHandler->validBasicAttribute('EpagesDir'), '\'EpagesDir\' is a valid basic attribute');
    
    ok(not ($PatchSystemInfoHandler->validBasicAttribute('EpagesFolder')), '\'EpagesFolder\' is not a valid basic attribute');
            
    ok($PatchSystemInfoHandler->existsBasicAttribute('ePagesTargetVersion'), 'key \'ePagesTargetVersion\' exists');

    ok(not ($PatchSystemInfoHandler->existsBasicAttribute('EpagesSharedDir')), 'key \'EpagesSharedDir\' does not exists');

    eval{
        ok(not $PatchSystemInfoHandler->existsBasicAttribute('AnyKey'), 'key \'AnyKey\' does not exists');
    };
    ok( ExistsError(), 'searching for key \'AnyKey\' throws error');

    ok($PatchSystemInfoHandler->virtualBasicAttribute('ePagesCurrentVersion'), '\'ePagesCurrentVersion\' is a virtual basic attribute');
    
    ok(not ($PatchSystemInfoHandler->virtualBasicAttribute('ePagesPastVersion')), '\'ePagesPastVersion\' is not a virtual basic attribute');
    
    ok($PatchSystemInfoHandler->lockedVersionKey('CartridgesDataPackageFile'), 'locked version key \'CartridgesDataPackageFile\' exists');
    
    ok(not ($PatchSystemInfoHandler->lockedVersionKey('CartridgesDataPackage')), '\'CartridgesDataPackage\' is not a locked version key');
    
    is($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion'), '2.1.3', 'key \'ePagesTargetVersion\' is correct');

    ok($PatchSystemInfoHandler->getPatchTag() eq 'patch_2_1_3', 'patch tag \'patch_2_12_3\' is correct');

    $PatchSystemInfoHandler->setBasicAttribute('ePagesTargetVersion', '2.1.2');
    
    isnt($PatchSystemInfoHandler->getBasicAttribute('ePagesTargetVersion'), '2.1.3', 'key \'ePagesTargetVersion\' is correctly changed');

    isnt($PatchSystemInfoHandler->getPatchTag(), 'patch_2_1_3', 'patch tag is not correct for ePagesTargetVersion \'2.1.2\'');
    
    is($PatchSystemInfoHandler->getPatchTag(), 'patch_2_1_2', 'patch tag \'patch_2_12_2\' is correct');
    
    $PatchSystemInfoHandler->setVersionAttribute('1.1.1', 'EpagesRepoUrl', '1');
    $PatchSystemInfoHandler->setVersionAttribute('1.1.2', 'EpagesRepoUrl', '0');

    is ($PatchSystemInfoHandler->getVersionAttribute('1.1.1', 'EpagesRepoUrl'), 1, 'version attribute for \'EpagesRepoUrl\' for version \'1.1.1\' correct');

    is ($PatchSystemInfoHandler->getVersionAttribute('1.1.2', 'EpagesRepoUrl'), 0, 'version attribute for \'EpagesRepoUrl\' for version \'1.1.2\' correct');

    #valid version
    my $VersionsToPatch = [
        '1.1.2',
        '1.1.3',
        '1.2.1',
        '1.2.2',
        '1.2.3',
        '2.1.1',
        '2.1.2',
        '2.1.3',
        '6.17',
        '6.17.9',
        '6.17.9.3',
        '6.17.9.3.1'   
    ];
    
    my $TestPath = ['1.1.1','1.1.2','1.1.3','1.2.1','1.2.2','1.2.3','2.1.1','2.1.2','2.1.3','6.17','6.17.9','6.17.9.3','6.17.9.3.1'];
    
    is (Dumper($PatchSystemInfoHandler->validVersionsToPatch()), Dumper($VersionsToPatch), 'versions to patch correct');
    
    is_deeply ($PatchSystemInfoHandler->validVersions, $TestPath, 'version path correct');

    ok ($PatchSystemInfoHandler->validVersion('1.2.2'), 'version \'1.2.2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validVersion('5.5.5')), 'version \'5.5.5\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch'), 'function \'API::Patch::PatchPrepareHandler::doPatch\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPopelHandler::doPatch')), 'function \'API::Patch::PatchPopelHandler::doPatch\' is not valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_universal'), 'function \'API::Patch::PatchPrepareHandler::doPatch_universal\' is valid');

    ok ( $PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_1_2_2'), 'function \'API::Patch::PatchPrepareHandler::doPatch_1_2_2\' is valid');

    ok ( not ($PatchSystemInfoHandler->validFunction('API::Patch::PatchPrepareHandler::doPatch_5_5_5')), 'function \'API::Patch::PatchPrepareHandler::doPatch_5_5_5\' is not valid');

    my $GroundFunctions = [
        'callSubHandlerWithVersionSources',
        'callSubHandlerWithVersionSourcesAsBackgroundProcess',
        'doPatch',
        'doPatch_1_1_1',
        'doPatch_1_1_2',
        'doPatch_1_1_3',
        'doPatch_1_2_1',
        'doPatch_1_2_2',
        'doPatch_1_2_3',
        'doPatch_2_1_1',
        'doPatch_2_1_2',
        'doPatch_2_1_3',
        'doPatch_6_17',
        'doPatch_6_17_9',
        'doPatch_6_17_9_3',
        'doPatch_6_17_9_3_1',        
        'doPatch_basic',
        'doPatch_checkPreCondition',
        'doPatch_universal',
        'toExecute',
        'useNow'
    ];

    is_deeply ( $PatchSystemInfoHandler->validGroundFunctions(), $GroundFunctions, 'ground functions correct');

    ok ($PatchSystemInfoHandler->validGroundFunction('doPatch_1_1_2'), '\'doPatch_1_1_2\' is a valid ground function');

    ok (not ($PatchSystemInfoHandler->validGroundFunction('install_1_1_2')), '\'install_1_1_2\' is not a valid ground function');
    
    my $Functions = [
          'API::Patch::PatchDownloadHandler::callSubHandlerWithVersionSources',
          'API::Patch::PatchDownloadHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchDownloadHandler::doPatch',
          'API::Patch::PatchDownloadHandler::doPatch_1_1_1',
          'API::Patch::PatchDownloadHandler::doPatch_1_1_2',
          'API::Patch::PatchDownloadHandler::doPatch_1_1_3',
          'API::Patch::PatchDownloadHandler::doPatch_1_2_1',
          'API::Patch::PatchDownloadHandler::doPatch_1_2_2',
          'API::Patch::PatchDownloadHandler::doPatch_1_2_3',
          'API::Patch::PatchDownloadHandler::doPatch_2_1_1',
          'API::Patch::PatchDownloadHandler::doPatch_2_1_2',
          'API::Patch::PatchDownloadHandler::doPatch_2_1_3',
          'API::Patch::PatchDownloadHandler::doPatch_6_17',
          'API::Patch::PatchDownloadHandler::doPatch_6_17_9',
          'API::Patch::PatchDownloadHandler::doPatch_6_17_9_3',
          'API::Patch::PatchDownloadHandler::doPatch_6_17_9_3_1',
          'API::Patch::PatchDownloadHandler::doPatch_basic',
          'API::Patch::PatchDownloadHandler::doPatch_checkPreCondition',
          'API::Patch::PatchDownloadHandler::doPatch_universal',
          'API::Patch::PatchDownloadHandler::toExecute',
          'API::Patch::PatchDownloadHandler::useNow',
          'API::Patch::PatchDownloadHandlerCartridgePacks::callSubHandlerWithVersionSources',
          'API::Patch::PatchDownloadHandlerCartridgePacks::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_1_1',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_1_2',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_1_3',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_2_1',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_2_2',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_1_2_3',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_2_1_1',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_2_1_2',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_2_1_3',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_6_17',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_6_17_9',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_6_17_9_3',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_6_17_9_3_1',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_basic',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_checkPreCondition',
          'API::Patch::PatchDownloadHandlerCartridgePacks::doPatch_universal',
          'API::Patch::PatchDownloadHandlerCartridgePacks::toExecute',
          'API::Patch::PatchDownloadHandlerCartridgePacks::useNow',
          'API::Patch::PatchDownloadHandlerLanguagePacks::callSubHandlerWithVersionSources',
          'API::Patch::PatchDownloadHandlerLanguagePacks::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_1_1',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_1_2',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_1_3',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_2_1',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_2_2',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_1_2_3',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_2_1_1',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_2_1_2',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_2_1_3',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_6_17',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_6_17_9',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_6_17_9_3',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_6_17_9_3_1',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_basic',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_checkPreCondition',
          'API::Patch::PatchDownloadHandlerLanguagePacks::doPatch_universal',
          'API::Patch::PatchDownloadHandlerLanguagePacks::toExecute',
          'API::Patch::PatchDownloadHandlerLanguagePacks::useNow',
          'API::Patch::PatchDownloadHandlerWebRootPacks::callSubHandlerWithVersionSources',
          'API::Patch::PatchDownloadHandlerWebRootPacks::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_1_1',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_1_2',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_1_3',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_2_1',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_2_2',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_1_2_3',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_2_1_1',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_2_1_2',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_2_1_3',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_6_17',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_6_17_9',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_6_17_9_3',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_6_17_9_3_1',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_basic',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_checkPreCondition',
          'API::Patch::PatchDownloadHandlerWebRootPacks::doPatch_universal',
          'API::Patch::PatchDownloadHandlerWebRootPacks::toExecute',
          'API::Patch::PatchDownloadHandlerWebRootPacks::useNow',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::callSubHandlerWithVersionSources',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_1_1',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_1_2',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_1_3',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_2_1',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_2_2',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_1_2_3',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_2_1_1',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_2_1_2',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_2_1_3',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_6_17',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_6_17_9',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_6_17_9_3',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_6_17_9_3_1',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_basic',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_checkPreCondition',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::doPatch_universal',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::toExecute',
          'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks::useNow',
          'API::Patch::PatchPrepareHandler::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandler::doPatch',
          'API::Patch::PatchPrepareHandler::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandler::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandler::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandler::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandler::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandler::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandler::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandler::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandler::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandler::doPatch_6_17',
          'API::Patch::PatchPrepareHandler::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandler::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandler::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandler::doPatch_basic',
          'API::Patch::PatchPrepareHandler::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandler::doPatch_universal',
          'API::Patch::PatchPrepareHandler::toExecute',
          'API::Patch::PatchPrepareHandler::useNow',
          'API::Patch::PatchPrepareHandlerVersionFolders::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerVersionFolders::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_universal',
          'API::Patch::PatchPrepareHandlerVersionFolders::toExecute',
          'API::Patch::PatchPrepareHandlerVersionFolders::useNow',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::toExecute',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::useNow',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_basic',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::doPatch_universal',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::toExecute',
          'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML::useNow',
          'API::Patch::PatchPrepareHandlerStoreTypes::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerStoreTypes::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_basic',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerStoreTypes::doPatch_universal',
          'API::Patch::PatchPrepareHandlerStoreTypes::toExecute',
          'API::Patch::PatchPrepareHandlerStoreTypes::useNow',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_6_17',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_basic',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::doPatch_universal',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::toExecute',
          'API::Patch::PatchPrepareHandler3rdPartyPrograms::useNow',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::toExecute',
          'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::useNow',
          'API::Patch::PatchPrepareHandlerDojo::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerDojo::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerDojo::doPatch',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_basic',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerDojo::doPatch_universal',
          'API::Patch::PatchPrepareHandlerDojo::toExecute',
          'API::Patch::PatchPrepareHandlerDojo::useNow',
          'API::Patch::PatchPrepareHandlerUIPacker::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerUIPacker::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_basic',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerUIPacker::doPatch_universal',
          'API::Patch::PatchPrepareHandlerUIPacker::toExecute',
          'API::Patch::PatchPrepareHandlerUIPacker::useNow',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_basic',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::doPatch_universal',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::toExecute',
          'API::Patch::PatchPrepareHandlerWebRootUpdater::useNow',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::callSubHandlerWithVersionSources',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_1_1',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_1_2',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_1_3',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_2_1',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_2_2',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_1_2_3',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_2_1_1',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_2_1_2',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_2_1_3',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_6_17',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_6_17_9',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_6_17_9_3',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_6_17_9_3_1',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_basic',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_checkPreCondition',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::doPatch_universal',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::toExecute',
          'API::Patch::PatchPrepareHandlerCompiledTemplates::useNow',
          'API::Patch::PatchPreinstallHandler::callSubHandlerWithVersionSources',
          'API::Patch::PatchPreinstallHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPreinstallHandler::doPatch',
          'API::Patch::PatchPreinstallHandler::doPatch_1_1_1',
          'API::Patch::PatchPreinstallHandler::doPatch_1_1_2',
          'API::Patch::PatchPreinstallHandler::doPatch_1_1_3',
          'API::Patch::PatchPreinstallHandler::doPatch_1_2_1',
          'API::Patch::PatchPreinstallHandler::doPatch_1_2_2',
          'API::Patch::PatchPreinstallHandler::doPatch_1_2_3',
          'API::Patch::PatchPreinstallHandler::doPatch_2_1_1',
          'API::Patch::PatchPreinstallHandler::doPatch_2_1_2',
          'API::Patch::PatchPreinstallHandler::doPatch_2_1_3',
          'API::Patch::PatchPreinstallHandler::doPatch_6_17',
          'API::Patch::PatchPreinstallHandler::doPatch_6_17_9',
          'API::Patch::PatchPreinstallHandler::doPatch_6_17_9_3',
          'API::Patch::PatchPreinstallHandler::doPatch_6_17_9_3_1',
          'API::Patch::PatchPreinstallHandler::doPatch_basic',
          'API::Patch::PatchPreinstallHandler::doPatch_checkPreCondition',
          'API::Patch::PatchPreinstallHandler::doPatch_universal',
          'API::Patch::PatchPreinstallHandler::toExecute',
          'API::Patch::PatchPreinstallHandler::useNow',
          'API::Patch::PatchInstallHandler::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandler::doPatch',
          'API::Patch::PatchInstallHandler::doPatch_1_1_1',
          'API::Patch::PatchInstallHandler::doPatch_1_1_2',
          'API::Patch::PatchInstallHandler::doPatch_1_1_3',
          'API::Patch::PatchInstallHandler::doPatch_1_2_1',
          'API::Patch::PatchInstallHandler::doPatch_1_2_2',
          'API::Patch::PatchInstallHandler::doPatch_1_2_3',
          'API::Patch::PatchInstallHandler::doPatch_2_1_1',
          'API::Patch::PatchInstallHandler::doPatch_2_1_2',
          'API::Patch::PatchInstallHandler::doPatch_2_1_3',
          'API::Patch::PatchInstallHandler::doPatch_6_17',
          'API::Patch::PatchInstallHandler::doPatch_6_17_9',
          'API::Patch::PatchInstallHandler::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandler::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandler::doPatch_basic',
          'API::Patch::PatchInstallHandler::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandler::doPatch_universal',
          'API::Patch::PatchInstallHandler::toExecute',
          'API::Patch::PatchInstallHandler::useNow',
          'API::Patch::PatchInstallHandlerCPAN::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerCPAN::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerCPAN::doPatch',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_6_17',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_basic',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerCPAN::doPatch_universal',
          'API::Patch::PatchInstallHandlerCPAN::toExecute',
          'API::Patch::PatchInstallHandlerCPAN::useNow',
          'API::Patch::PatchInstallHandlerCartridgeFolder::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerCartridgeFolder::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_6_17',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_basic',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerCartridgeFolder::doPatch_universal',
          'API::Patch::PatchInstallHandlerCartridgeFolder::toExecute',
          'API::Patch::PatchInstallHandlerCartridgeFolder::useNow',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_6_17',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_basic',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::doPatch_universal',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::toExecute',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart::useNow',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_6_17',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_basic',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::doPatch_universal',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::toExecute',
          'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML::useNow',
          'API::Patch::PatchInstallHandlerInformationCollector::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerInformationCollector::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_6_17',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_basic',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerInformationCollector::doPatch_universal',
          'API::Patch::PatchInstallHandlerInformationCollector::toExecute',
          'API::Patch::PatchInstallHandlerInformationCollector::useNow',
          'API::Patch::PatchInstallHandlerSite::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerSite::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerSite::doPatch',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerSite::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerSite::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerSite::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerSite::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerSite::doPatch_6_17',
          'API::Patch::PatchInstallHandlerSite::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerSite::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerSite::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerSite::doPatch_basic',
          'API::Patch::PatchInstallHandlerSite::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerSite::doPatch_universal',
          'API::Patch::PatchInstallHandlerSite::toExecute',
          'API::Patch::PatchInstallHandlerSite::useNow',
          'API::Patch::PatchInstallHandlerASPoolDB::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerASPoolDB::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_6_17',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_basic',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerASPoolDB::doPatch_universal',
          'API::Patch::PatchInstallHandlerASPoolDB::toExecute',
          'API::Patch::PatchInstallHandlerASPoolDB::useNow',
          'API::Patch::PatchInstallHandlerStores::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerStores::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerStores::doPatch',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerStores::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerStores::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerStores::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerStores::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerStores::doPatch_6_17',
          'API::Patch::PatchInstallHandlerStores::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerStores::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerStores::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerStores::doPatch_basic',
          'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerStores::doPatch_universal',
          'API::Patch::PatchInstallHandlerStores::toExecute',
          'API::Patch::PatchInstallHandlerStores::useNow',
          'API::Patch::PatchInstallHandlerStoreTypes::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerStoreTypes::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_6_17',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_basic',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerStoreTypes::doPatch_universal',
          'API::Patch::PatchInstallHandlerStoreTypes::toExecute',
          'API::Patch::PatchInstallHandlerStoreTypes::useNow',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_6_17',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_basic',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::doPatch_universal',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::toExecute',
          'API::Patch::PatchInstallHandlerUpdateCompiledTemplates::useNow',
          'API::Patch::PatchInstallHandlerAppServerRestart::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerAppServerRestart::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_6_17',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_basic',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerAppServerRestart::doPatch_universal',
          'API::Patch::PatchInstallHandlerAppServerRestart::toExecute',
          'API::Patch::PatchInstallHandlerAppServerRestart::useNow',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_6_17',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_basic',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::doPatch_universal',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::toExecute',
          'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart::useNow',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_6_17',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_basic',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::doPatch_universal',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::toExecute',
          'API::Patch::PatchInstallHandlerFlexFeatureRefs::useNow',
          'API::Patch::PatchInstallHandlerFlexShopTypes::callSubHandlerWithVersionSources',
          'API::Patch::PatchInstallHandlerFlexShopTypes::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_1',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_2',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_1_3',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_1',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_2',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_1_2_3',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_1',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_2',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_2_1_3',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_6_17',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_6_17_9',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_6_17_9_3',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_6_17_9_3_1',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_basic',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_checkPreCondition',
          'API::Patch::PatchInstallHandlerFlexShopTypes::doPatch_universal',
          'API::Patch::PatchInstallHandlerFlexShopTypes::toExecute',
          'API::Patch::PatchInstallHandlerFlexShopTypes::useNow',
          'API::Patch::PatchPostInstallHandler::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandler::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandler::doPatch',
          'API::Patch::PatchPostInstallHandler::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandler::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandler::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandler::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandler::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandler::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandler::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandler::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandler::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandler::doPatch_6_17',
          'API::Patch::PatchPostInstallHandler::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandler::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandler::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandler::doPatch_basic',
          'API::Patch::PatchPostInstallHandler::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandler::doPatch_universal',
          'API::Patch::PatchPostInstallHandler::toExecute',
          'API::Patch::PatchPostInstallHandler::useNow',
          'API::Patch::PatchPostInstallHandlerWebroot::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerWebroot::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerWebroot::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerWebroot::toExecute',
          'API::Patch::PatchPostInstallHandlerWebroot::useNow',
          'API::Patch::PatchPostInstallHandlerCleanUp::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerCleanUp::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerCleanUp::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerCleanUp::toExecute',
          'API::Patch::PatchPostInstallHandlerCleanUp::useNow',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::toExecute',
          'API::Patch::PatchPostInstallHandlerSynchronizeSite::useNow',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::toExecute',
          'API::Patch::PatchPostInstallHandlerRemoveCartridges::useNow',
          'API::Patch::PatchPostInstallHandlerConfigFiles::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerConfigFiles::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerConfigFiles::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerConfigFiles::toExecute',
          'API::Patch::PatchPostInstallHandlerConfigFiles::useNow',
          'API::Patch::PatchPostInstallHandlerServices::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerServices::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerServices::doPatch',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerServices::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerServices::toExecute',
          'API::Patch::PatchPostInstallHandlerServices::useNow',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::toExecute',
          'API::Patch::PatchPostInstallHandlerRemoteSearch::useNow',
          'API::Patch::PatchPostInstallHandlerMultiStore::callSubHandlerWithVersionSources',
          'API::Patch::PatchPostInstallHandlerMultiStore::callSubHandlerWithVersionSourcesAsBackgroundProcess',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_1_1',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_1_2',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_1_3',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_2_1',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_2_2',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_1_2_3',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_2_1_1',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_2_1_2',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_2_1_3',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_6_17',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_6_17_9',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_6_17_9_3',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_6_17_9_3_1',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_basic',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_checkPreCondition',
          'API::Patch::PatchPostInstallHandlerMultiStore::doPatch_universal',
          'API::Patch::PatchPostInstallHandlerMultiStore::toExecute',
          'API::Patch::PatchPostInstallHandlerMultiStore::useNow'
    ];
    
    my $PatchHandler = [
        'API::Patch::PatchDownloadHandler',
        'API::Patch::PatchDownloadHandlerCartridgePacks',
        'API::Patch::PatchDownloadHandlerLanguagePacks',
        'API::Patch::PatchDownloadHandlerWebRootPacks',
        'API::Patch::PatchDownloadHandlerCompiledTemplatesPacks',
        'API::Patch::PatchPrepareHandler',
        'API::Patch::PatchPrepareHandlerVersionFolders',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeXML',
        'API::Patch::PatchPrepareHandlerStoreTypes',
        'API::Patch::PatchPrepareHandler3rdPartyPrograms',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter',
        'API::Patch::PatchPrepareHandlerDojo',
        'API::Patch::PatchPrepareHandlerUIPacker',
        'API::Patch::PatchPrepareHandlerWebRootUpdater',
        'API::Patch::PatchPrepareHandlerCompiledTemplates',
        'API::Patch::PatchPreinstallHandler',
        'API::Patch::PatchInstallHandler',
        'API::Patch::PatchInstallHandlerCPAN',
        'API::Patch::PatchInstallHandlerCartridgeFolder',
        'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeDirPriorPart',
        'API::Patch::PatchInstallHandlerCartridgeFolderPatchCartridgeXML',
        'API::Patch::PatchInstallHandlerInformationCollector',
        'API::Patch::PatchInstallHandlerSite',
        'API::Patch::PatchInstallHandlerASPoolDB',
        'API::Patch::PatchInstallHandlerStores',
        'API::Patch::PatchInstallHandlerStoreTypes',
        'API::Patch::PatchInstallHandlerUpdateCompiledTemplates',
        'API::Patch::PatchInstallHandlerAppServerRestart',
        'API::Patch::PatchInstallHandlerPatchCartridgeDirFinalPart',
        'API::Patch::PatchInstallHandlerFlexFeatureRefs',
        'API::Patch::PatchInstallHandlerFlexShopTypes',
        'API::Patch::PatchPostInstallHandler',
        'API::Patch::PatchPostInstallHandlerWebroot',
        'API::Patch::PatchPostInstallHandlerCleanUp',
        'API::Patch::PatchPostInstallHandlerSynchronizeSite',
        'API::Patch::PatchPostInstallHandlerRemoveCartridges',
        'API::Patch::PatchPostInstallHandlerConfigFiles',
        'API::Patch::PatchPostInstallHandlerServices',
        'API::Patch::PatchPostInstallHandlerRemoteSearch',
        'API::Patch::PatchPostInstallHandlerMultiStore'
    ];
        
    ok ($PatchSystemInfoHandler->validHandler('PatchDownloadHandler'), 'handler \'PatchDownloadHandler\' is valid');
    
    ok ( not ($PatchSystemInfoHandler->validHandler('PatchEPagesHandler')), 'handler \'PatchEPagesHandler\' is not valid');
        
    is_deeply ($PatchSystemInfoHandler->validHandlers(), $PatchHandler, 'handlers correct');
    
    is ( $PatchSystemInfoHandler->getHandlerBriefing('PatchInstallHandler'), 'patch cpan, aspooldb, site, stores and file system','handler briefing correct');
    
    isnt ($PatchSystemInfoHandler->getHandlerBriefing('PatchInstallHandler'), 'prepare patch files', 'handler briefing for \'PatchInstallHandler\' not correct');

    is ( $PatchSystemInfoHandler->getHandlerNumeration('PatchInstallHandler'),'4', 'handler numeration correct');
    
    isnt ($PatchSystemInfoHandler->getHandlerNumeration('PatchPrepareHandler'), 1, 'get handler numeration for \'PatchPrepareHandler\' not correct');

    is_deeply ($PatchSystemInfoHandler->validFunctions(), $Functions, 'functions correct');
    
    ok ($PatchSystemInfoHandler->validInstallerFunction('patch'), '\'patch\' is a valid installer function');

    ok (not ($PatchSystemInfoHandler->validInstallerFunction('download')), '\'download\' is not a valid installer function');

    ok (not ($PatchSystemInfoHandler->isLastVersion('1.2.2')), 'version \'1.2.2\' is not last version');

    ok ($PatchSystemInfoHandler->isLastVersion('6.17.9.3.1'), 'version \'2.1.3\' is last version');

    ok (not ($PatchSystemInfoHandler->isFirstVersion('1.2.2')), 'version \'1.2.2\' is not first version');

    ok ($PatchSystemInfoHandler->isFirstVersion('1.1.1'), 'version \'1.1.1\' is first version');

    ok (($PatchSystemInfoHandler->firstVersion() eq '1.1.1'), 'first version eq \'1.1.1\'');

    ok (($PatchSystemInfoHandler->lastVersion() eq '6.17.9.3.1'), 'last version eq \'6.17.9.3.1\'');

    ok (($PatchSystemInfoHandler->nextVersion('1.2.3') eq '2.1.1'), 'next version from \'1.2.3\' is version \'2.1.1\'');

    eval{$PatchSystemInfoHandler->nextVersion('6.17.9.3.1')};
    ok( ExistsError(), 'next version from last version throws an error');

    is ($PatchSystemInfoHandler->previousVersion('2.1.1'), '1.2.3', 'previous version from \'2.1.1\' is version \'1.2.3\'');

    eval{$PatchSystemInfoHandler->previousVersion('1.1.1')};
    ok( ExistsError(), 'previous version from first version throws an error');

    ok($PatchSystemInfoHandler->isTargetVersion('2.1.2'), '\'2.1.3\' is target version');
       
    ok(not ($PatchSystemInfoHandler->isTargetVersion('2.1.1')), '\'2.1.1\' is not the target version');
    
    is($PatchSystemInfoHandler->getVersionOrder('2.1.1'), 7, 'version order id for \'2.1.1\' is \'7\'');

    isnt ($PatchSystemInfoHandler->getVersionOrder('2.1.2'), 7, 'version order id for \'2.1.2\' is not \'7\'');

    is ($PatchSystemInfoHandler->getMainPartOfVersion('6.17.9.3'), '6.17.9', 'get main part of version correct');

    isnt($PatchSystemInfoHandler->getMainPartOfVersion('1.2.1'), '1.2', 'main part of version \'1.2.1\' is not \'1.2\'');

    is ($PatchSystemInfoHandler->getPartFromVersion('2.1.3', 0), 2, 'part \'0\' of version \'2.1.3\' is 2');

    is ($PatchSystemInfoHandler->getPartFromVersion('2.1.3', 1), 1, 'part \'1\' of version \'2.1.3\' is 1');

    is ($PatchSystemInfoHandler->getPartFromVersion('2.1.3', 2), 3, 'part \'2\' of version \'2.1.3\' is 3');
    
    isnt ($PatchSystemInfoHandler->getPartFromVersion('2.1.3', 2), 1, 'part \'2\' of version \'2.1.3\' isnt 1');
    
    isnt ($PatchSystemInfoHandler->getPartFromVersion('2.1.3', 1), 2, 'part \'1\' from version \'2.1.3\' isnt 2');
    
    ok (
        not ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1')),
        'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not started for version \'2.1.1\' on host \'alphatesthost1\''
    );

    $PatchSystemInfoHandler->registerStartedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic', 'alphatesthost1');
    sleep(3);
    $PatchSystemInfoHandler->registerCompletedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic','alphatesthost1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch', 'alphatesthost1');
    sleep(4);

    ok (
        ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch', 'alphatesthost1')),
        'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now started for version \'2.1.1\' on host \'alphatesthost1\''
    );

    ok (
        not ($PatchSystemInfoHandler->isPatchFunctionStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','testhost1')),
        'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not started for version \'2.1.1\' on host \'testhost1\''
    );
    
    ok ($PatchSystemInfoHandler->isPatchHandlerStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir','alphatesthost1'), '\'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' is started');
    
    ok (not ($PatchSystemInfoHandler->isPatchHandlerStarted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir','testhost1')), '\'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' is not started');

    ok (
        not ($PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1')),
        'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' not completed for version \'2.1.1\' on host \'alphatesthost1\''
    );

    $PatchSystemInfoHandler->registerCompletedPatchFunction('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1');

    is ($PatchSystemInfoHandler->getPatchHandlerDuration('PatchPrepareHandlerVersionFoldersPatchCartridgeDir', '2.1.1', 'alphatesthost1'), 4,'patch handler duration correct');

    is ($PatchSystemInfoHandler->getPatchHandlerFunctionDuration('PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'doPatch_basic', '2.1.1', 'alphatesthost1'), 3, 'patch handler function duration correct');

    is ($PatchSystemInfoHandler->getSpendedHandlerTimeMessage('PatchPrepareHandlerVersionFoldersPatchCartridgeDir', '2.1.1', '2.1.1', 'alphatesthost1'), '4 s', 'spended handler time message correct');

    ok (
        $PatchSystemInfoHandler->isPatchFunctionCompleted('2.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1'),
        'function \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch\' now completed for version \'2.1.1\''
    );

    eval{
        $PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'alphatesthost1');
    };
    ok( ExistsError(), 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has no current version on host \'alphatesthost1\'');

    $PatchSystemInfoHandler->bindHandlerToHost('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir','alphatesthost1');

    eval{
        $PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'alphatesthost1');
    };
    ok( ExistsError(), 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has no current version on host \'alphatesthost1\'');

    ok(not ($PatchSystemInfoHandler->existsCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'alphatesthost1')), 'handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' has no current version on host \'alphatesthost1\'');

    eval{
        $PatchSystemInfoHandler->setCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'testhost1', '2.1.1');
    };
    ok( ExistsError(), 'cant set version  for handler \'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir\' on unbinded host \'testhost1\'');

    $PatchSystemInfoHandler->setCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'alphatesthost1', '2.1.1');

    is(
        $PatchSystemInfoHandler->getCurrentHandlerVersion('API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir', 'alphatesthost1'),
        '2.1.1',
        'correct handler version'
    );

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.3', '2.1.2'), 'version \'1.1.3\' is lower than version \'2.1.2\'');
    
    ok ($PatchSystemInfoHandler->versionLowerOrEqualThan('1.1.3', '2.1.2'), 'version \'1.1.3\' is lower or equal than version \'2.1.2\'');
    
    ok ($PatchSystemInfoHandler->versionLowerOrEqualThan('2.1.2', '2.1.2'), 'version \'1.1.3\' is lower or equal than version \'2.1.2\'');
    
    ok (not ($PatchSystemInfoHandler->versionLowerOrEqualThan('2.1.2','1.1.3')), 'version \'2.1.2\' is lower or equal than version \'1.1.3\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('2.1.2', '1.1.3')), 'version \'2.1.2\' is not lower than version \'1.1.3\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.2.1', '1.1.3'), 'not existing version \'1.1.2.1\' is lower than existing version \'1.1.3\'');

    ok ($PatchSystemInfoHandler->versionLowerThan('1.1.2.1', '1.1.2.2'), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.2.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3.1', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.3.1\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than existing version \'1.1.3\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.2.2', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.2.2\'');

    ok (not ($PatchSystemInfoHandler->versionLowerThan('1.1.3.1', '1.1.2.1')), 'not existing version \'1.1.2.1\' is lower than not existing version \'1.1.3.1\'');

    ok ($PatchSystemInfoHandler->versionGreaterThan('2.1.2', '1.1.3'), 'version \'2.1.2\' is greater than version \'1.1.3\'');
    
    ok ($PatchSystemInfoHandler->versionGreaterOrEqualThan('2.1.2', '1.1.3'), 'version \'2.1.2\' is greater or equal than version \'1.1.3\'');
    
    ok ($PatchSystemInfoHandler->versionGreaterOrEqualThan('1.1.3', '1.1.3'), 'version \'1.1.3\' is greater or equal than version \'1.1.3\'');
    
    ok (not ($PatchSystemInfoHandler->versionGreaterOrEqualThan('1.1.3', '2.1.2')), 'version \'1.1.3\' is greater or equal than version \'2.1.2\'');

    ok (not ($PatchSystemInfoHandler->versionGreaterThan('1.1.3', '2.1.2')), 'version \'1.1.3\' is not greater than version \'2.1.2\'');

    is ($PatchSystemInfoHandler->getMainPartOfVersion('6.17.9.3'), '6.17.9' , 'main part of version \'6.17.9.3\' is \'6.17.9\'');
    
    #eval{$PatchSystemInfoHandler->versionGreaterThan('6.6.6', '1.1.3')};
    #ok( ExistsError(), 'check invalid number throws error');

    #eval{$PatchSystemInfoHandler->versionGreaterThan('1.1.3', '6.6.6')};
    #ok( ExistsError(), 'check invalid number throws error');

    #tests for cartridges version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');
    
    eval{$PatchSystemInfoHandler->getEpagesStoreTypeVersionFolder('1.1.3')};
    ok( ExistsError(), 'get store type version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('EpagesCartridgesVersionDir','C:\epages\Cartridges');
    my $EpagesWebroot = RealPath($ENV{'EPAGES_WEBROOT'});
    $PatchSystemInfoHandler->setBasicAttribute('EpagesWebrootDir', $EpagesWebroot);
    
    is ( $PatchSystemInfoHandler->getEpagesStoreTypeVersionFolder('1.1.3'), $EpagesWebroot . '/StoreTypes/1.1.3' , "store type version folder for version '1.1.3' correct '$EpagesWebroot/StoreTypes/1.1.3'");

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct \'C:/epages/Cartridges/1.1.3\'');

    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does not exist for version \'1.1.3\' without set manualy');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'D:/elsewhere/Test/1.1.3');

    ok (($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','CartridgesVersionFolder')), 'version attribute \'CartridgesVersionFolder\' does now exists exist for version \'1.1.3\'');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'D:/elsewhere/Test/1.1.3', 'cartridges version folder for version \'1.1.3\' correct changed \'D:/elsewhere/Test/1.1.3\'');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('2.1.1'), 'C:/epages/Cartridges/2.1.1', 'cartridges version folder for version \'2.1.1\' stay correct by default \'C:/epages/Cartridges/2.1.1\'');

    $PatchSystemInfoHandler->setCartridgesVersionFolder('1.1.3', 'C:/epages/Cartridges/1.1.3');

    is ( $PatchSystemInfoHandler->getCartridgesVersionFolder('1.1.3'), 'C:/epages/Cartridges/1.1.3', 'cartridges version folder for version \'1.1.3\' correct setting back to \'C:/epages/Cartridges/1.1.3\'');
    
    is ( $PatchSystemInfoHandler->getCartridgesVersionFolderFileList('1.1.1'), "$PatchDir/Data/DataPackageFileLists/cartridges-1.1.1.txt", 'cartridges version folder file list correct');

    ok (not ($PatchSystemInfoHandler->existsCartridgesVersionFolderFileList('1.1.2')), 'cartridge version folder filelist for version \'1.1.2\' does not exist');

    #patch event
    ok (not($PatchSystemInfoHandler->existsPatchEvent('Testing')), 'no patch event exists');

    my $PatchEventStartTime = $PatchSystemInfoHandler->getCurrentTime();
    $PatchSystemInfoHandler->registerPatchEventTime('Testing');

    ok ($PatchSystemInfoHandler->existsPatchEvent('Testing'), 'patch event \'Testing\' exists now');

    ok ($PatchSystemInfoHandler->getAllPatchEvents('Testing'), 'all patch events are correct');

    is ($PatchSystemInfoHandler->getPatchEventTime('Testing'), $PatchEventStartTime, 'patch event time is correct');

    #test compiled templates
    is ( $PatchSystemInfoHandler->getPatchCompiledTemplatesPackFile('2.1.1'), '/srv/epages/eproot/Shared/Patches/CtmplPacks/epages-2.1.1--compiled-templates.tar.gz', 'compiled templates pack file correct');
    isnt ( $PatchSystemInfoHandler->getPatchCompiledTemplatesPackFile('2.1.2', 'en-US'), '/srv/epages/eproot/Shared/Patches/CtmplPacks/epages-2.1.1--compiled-templates.tar.gz', 'compiled templates pack file is not correct for version \'2.1.2\'');

    is ( $PatchSystemInfoHandler->getPatchCompiledTemplatesUnpackDir('2.1.1'), $PatchExtractedDataPackDir . "/ExtractedCtmplPacks.v/2.1.1", 'compiled templates directory correct');

    is ( $PatchSystemInfoHandler->getEpagesCompiledTemplatesVersionDir('2.1.1'), '/srv/epages/eproot/Shared/Static/Templates.d/2.1.1', 'compiled template version dir correct');

    #tests for patch cartridges version folder
    ok (not ($PatchSystemInfoHandler->existsVersionAttribute('1.1.3','PatchCartridgesVersionFolder')), 'version attribute \'PatchCartridgesVersionFolder\' does not exist for version \'1.1.3\'');

    eval{$PatchSystemInfoHandler->getPatchCartridgesVersionFolder('1.1.3')};
    ok( ExistsError(), 'get version dir without initialization throws error');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesVersionDir','C:\Temp\patch_x_y_z\Cartridges.v');

    is ( $PatchSystemInfoHandler->getPatchCartridgesVersionFolder('1.1.3'), 'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3', 'patch cartridges version folder for version \'1.1.3\' correct \'C:/Temp/patch_x_y_z/Cartridges.v/1.1.3\'');

    is ( $PatchSystemInfoHandler->getPatchCartridgesVersionFolder('2.1.1'), 'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1', 'patch cartridges version folder for version \'2.1.1\' correct by default \'C:/Temp/patch_x_y_z/Cartridges.v/2.1.1\'');

    $PatchSystemInfoHandler->setBasicAttribute('PatchWebrootVersionDir', 'C:/Temp/patch_x_y_z/Shared/Webroot.v');

    is ($PatchSystemInfoHandler->getCartridgesVersionFolderFileList('2.1.2'), "$PatchDir/Data/DataPackageFileLists/cartridges-2.1.2.txt", 'CartridgesVersionFolderFileList for version \'2.1.2\' is correct');
    
    ok (not ($PatchSystemInfoHandler->existsCartridgesVersionFolderFileList('1.1.2')), 'cartridge version folder filelist for version \'1.1.2\' does not exist');
    
    #tests for patch webroot version folder
    is ( $PatchSystemInfoHandler->getPatchWebrootVersionFolder('1.1.3'), 'C:/Temp/patch_x_y_z/Shared/Webroot.v/1.1.3', 'patch webroot version folder for version \'1.1.3\' correct \'C:/Temp/patch_x_y_z/Shared/Webroot.v/1.1.3\'');

    is ($PatchSystemInfoHandler->getPatchExtractedWebRootPackVersionFolder('2.1.1'), "$PatchDir/ExtractedDataPacks/ExtractedWebRootPacks.v/2.1.1", 'patch extracted webroot pack version folder ok');

    is ($PatchSystemInfoHandler->getPatchExtractedWebRootPackVersionFolderDeleteList('2.1.1', 'StoreType7'),"$PatchDir/ExtractedDataPacks/ExtractedWebRootPacks.v/2.1.1/epages-2.1.1-StoreType7-deleted-files.txt",'patch extracted webroot pack version folder delete list is correct for version \'2.1.1\'');

    is ($PatchSystemInfoHandler->getPatchWebRootPackFile('2.1.1'), '/srv/epages/eproot/Shared/Patches/WebRootPacks/epages-2.1.1-storetype-changes.tar.gz', 'patch webroot pack file correct');

    
    #language pack version folder
    is ($PatchSystemInfoHandler->getLanguagePackVersionFolderFileList('1.1.2', 'en-US'), "$PatchDir/Data/DataPackageFileLists/language-en-US-1.1.2.txt", 'language pack version folder file list correct');

    ok (not ($PatchSystemInfoHandler->existsLanguagePackVersionFolderFileList('1.1.2', 'en-US')), 'language pack version folder file list for version \'1.1.2\' and language \'en-US\' does not exists');
      
    is ($PatchSystemInfoHandler->getLanguagePackFile('1.1.1', 'en-US'), '/srv/epages/eproot/Shared/Patches/LanguagePacks/langpack-en-US-epages11p1.tar.gz' ,'language pack file is correct'); 

    is ($PatchSystemInfoHandler->getLanguagePackFolder('2.1.3', 'en-US'), '/srv/epages/eproot/Shared/Patches/LanguagePacks/langpack-en-US-epages21p3', 'language pack folder correct');

    is ($PatchSystemInfoHandler->getCartridgeLanguageFolder('2.1.2', 'en-US'), '/srv/epages/eproot/Cartridges/.lang/2.1.2/en-US', 'cartridge language folder correct');

    #test storetypes
    ok (not ($PatchSystemInfoHandler->existsStoreType('StoreType1')), 'store type \'StoreType1\' does not exist');

    $PatchSystemInfoHandler->addStoreType('StoreType1');

    ok ($PatchSystemInfoHandler->existsStoreType('StoreType1'), 'store type \'StoreType1\' now exist');

    $PatchSystemInfoHandler->addStoreType('StoreType2');
    $PatchSystemInfoHandler->addStoreType('StoreType6');
    $PatchSystemInfoHandler->addStoreType('StoreType7');
    $PatchSystemInfoHandler->addStoreType('StoreType5');
    $PatchSystemInfoHandler->addStoreType('StoreType3');
    $PatchSystemInfoHandler->addStoreType('StoreType4');
    $PatchSystemInfoHandler->addStoreType('AAA');

    my $aStoreTypes = [
      'AAA',
      'StoreType1',
      'StoreType2',
      'StoreType3',
      'StoreType4',
      'StoreType5',
      'StoreType6',
      'StoreType7'
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypes(), $aStoreTypes, 'store types correct');

    ok(not($PatchSystemInfoHandler->existsStoreTypeCartridgeVersion('1.1.1','StoreType1','DE_EPAGES::CartridgeXY')), 'cartridge \'DE_EPAGES::CartridgeXY\' dont exists in store type \'StoreType1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXY', 7);

    ok($PatchSystemInfoHandler->existsStoreTypeCartridgeVersion('1.1.1','StoreType1','DE_EPAGES::CartridgeXY'), 'cartridge \'DE_EPAGES::CartridgeXY\' now exists in store type \'StoreType1\' for version \'1.1.1\'');
 
    ok($PatchSystemInfoHandler->existsStoreTypeCartridge('StoreType1', 'DE_EPAGES::CartridgeXY'), 'cartridge \'DE_EPAGES::CartridgeXY\' exists in store type \'StoreType1\'');
  
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXB', 3);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXG', 4);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXH', 5);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXZ', 8);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXA', 2);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType1', 'DE_EPAGES::CartridgeXK', 6);

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType2', 'DE_EPAGES::CartridgeAZ', 1);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.1','StoreType3', 'DE_EPAGES::CartridgeZZ', 1);

    $PatchSystemInfoHandler->addStore('TestStore1','StoreType1');
    $PatchSystemInfoHandler->addStore('TestStore2','StoreType1');
    $PatchSystemInfoHandler->addStore('TestStore3','StoreType1');

    $PatchSystemInfoHandler->addStore('TestStore4','StoreType2');
    $PatchSystemInfoHandler->addStore('TestStore5','StoreType2');
    $PatchSystemInfoHandler->addStore('TestStore6','StoreType2');

    ok($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeXA'), 'cartridge \'DE_EPAGES::CartridgeXA\' is in a store');
    ok($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeAZ'), 'cartridge \'DE_EPAGES::CartridgeAZ\' is in a store');
    ok(not($PatchSystemInfoHandler->isCartridgeInOneStore('DE_EPAGES::CartridgeZZ')),'cartridge \'DE_EPAGES::CartridgeZZ\' is not in a store');

    $PatchSystemInfoHandler->addStore('AAA','StoreType1');
    $PatchSystemInfoHandler->addStore('BBB','StoreType2');

    ok( $PatchSystemInfoHandler->isCartridgeInStore('TestStore1', 'DE_EPAGES::CartridgeXB') , 'cartridge \'DE_EPAGES::CartridgeXB\' is in store \'TestStore1\'');

    ok( not ($PatchSystemInfoHandler->isCartridgeInStore('TestStore1', 'DE_EPAGES::CartridgeAZ')) , 'cartridge \'DE_EPAGES::CartridgeAZ\' is not in store \'TestStore1\'');

    my $aAllCartridges = [
        'DE_EPAGES::CartridgeAZ',
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXB',
        'DE_EPAGES::CartridgeXG',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXY',
        'DE_EPAGES::CartridgeXZ',
        'DE_EPAGES::CartridgeZZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getAllCartridges(), $aAllCartridges, 'all cartridges correct');

    ok(($PatchSystemInfoHandler->isCartridgeUsingStandardVersions('DE_EPAGES::CartridgeAZ')), 'cartridge use standard versions');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeAZ', 0);

    ok(not ($PatchSystemInfoHandler->isCartridgeUsingStandardVersions('DE_EPAGES::CartridgeAZ')), 'cartridge use standard versions');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeAZ', 1);


    my $aCartridgesInVersion = [
        'DE_EPAGES::CartridgeAZ',
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXB',
        'DE_EPAGES::CartridgeXG',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXY',
        'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getCartridgesInVersion('1.1.1'), $aCartridgesInVersion, 'correct cartridges in version \'1.1.1\'');

    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXH', 1);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXZ', 2);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXA', 3);
    $PatchSystemInfoHandler->addStoreTypeCartridge('1.1.2','StoreType1', 'DE_EPAGES::CartridgeXK', 4);

    $aCartridgesInVersion = [
        'DE_EPAGES::CartridgeXA',
        'DE_EPAGES::CartridgeXH',
        'DE_EPAGES::CartridgeXK',
        'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getCartridgesInVersion('1.1.2'), $aCartridgesInVersion, 'correct cartridges in version \'1.1.2\'');
    
    #test unit store type
    my $aBusinessUnitStores = [
        'AAA',
        'BBB',
        'TestStore1',
        'TestStore2',
        'TestStore3',
        'TestStore4',
        'TestStore5',
        'TestStore6',
    ];

    ok ($PatchSystemInfoHandler->isBusinessUnitStoreType('site'), '\'site\' is a business unit store type');

    ok (not ($PatchSystemInfoHandler->isBusinessUnitStoreType('epagesj')), '\'epagesj\' is not a business unit store type');

    is_deeply ($PatchSystemInfoHandler->getBusinessUnitsStoreTypes(), $aStoreTypes , 'business unit store types correct');

    is_deeply ($PatchSystemInfoHandler->getBusinessUnitsStores(), $aBusinessUnitStores, 'business unit stores correct');

    #stores
    ok( $PatchSystemInfoHandler->existsStore('TestStore1') , 'Store \'TestStore1\' exists');
    ok( not ($PatchSystemInfoHandler->existsStore('TestStoreXY')) , 'Store \'TestStore2\' not exists');

    my $aStores = [
        'AAA',
        'BBB',
        'TestStore1',
        'TestStore2',
        'TestStore3',
        'TestStore4',
        'TestStore5',
        'TestStore6',
    ];
    is_deeply( $PatchSystemInfoHandler->getStores(), $aStores, 'stores correct');

    $aStores = [
        'AAA',
        'TestStore1',
        'TestStore2',
        'TestStore3',
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypeStores('StoreType1'), $aStores, 'stores corrects for \'StoreType1\'');

    $aStores = [
        'BBB',
        'TestStore4',
        'TestStore5',
        'TestStore6',
    ];
    is_deeply( $PatchSystemInfoHandler->getStoreTypeStores('StoreType2'), $aStores, 'stores corrects for \'StoreType2\'');

    is($PatchSystemInfoHandler->getStoreStoreType('AAA'), 'StoreType1', 'store have correct store type');
    is($PatchSystemInfoHandler->getStoreStoreType('BBB'), 'StoreType2', 'store have correct store type');

    ok(not ($PatchSystemInfoHandler->isStoreStarted('1.1.1','AAA')), 'store is not started to patch yet');
    ok(not ($PatchSystemInfoHandler->isStoreInProcess('AAA')), 'store is not in process yet');

    $PatchSystemInfoHandler->registerStartedStore('1.1.1','AAA', '1234567890');

    ok(not ($PatchSystemInfoHandler->allStoreStoresCompleted('1.1.1')), 'not all stores are completed');

    ok($PatchSystemInfoHandler->getProcessIDForStartedStore('AAA') eq '1234567890', 'correct processid');
    ok($PatchSystemInfoHandler->isStoreInProcess('AAA'), 'store is now in process');
    ok($PatchSystemInfoHandler->isStoreStarted('1.1.1','AAA'), 'store is now started to patch');
    ok(not ($PatchSystemInfoHandler->isStoreCompleted('1.1.1','AAA')), 'store is not complete patched yet');
    ok($PatchSystemInfoHandler->existsStoreProcessByStore('AAA'), 'process exists for started store');

    eval{$PatchSystemInfoHandler->registerCompletedStore('1.1.1','AAA')};
    ok( ExistsError(), 'cant register completed store because of unfinished cartridges');

    ok(not ($PatchSystemInfoHandler->isStoreCartridgeStarted('1.1.1','AAA','DE_EPAGES::CartridgeXA')), 'store cartridge is not started');
    ok(not ($PatchSystemInfoHandler->isStoreCartridgeCompleted('1.1.1','AAA','DE_EPAGES::CartridgeXA')), 'store cartridge is not completed');

    eval{$PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA')};
    ok( ExistsError(), 'cant register completed store cartridge because it is not started yet');

    #started store cartridges
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXZ');

    eval{$PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeAZ')};
    ok( ExistsError(), 'cant register completed store cartridge because doesnt contain the cartrige');

    ok($PatchSystemInfoHandler->isStoreCartridgeStarted('1.1.1','AAA','DE_EPAGES::CartridgeXA'), 'store cartridge is now started');


    #complete cartridges
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXA');

    ok($PatchSystemInfoHandler->isStoreCartridgeCompleted('1.1.1','AAA','DE_EPAGES::CartridgeXA'), 'store cartridge is now completed');

    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXY');

    ok(not ($PatchSystemInfoHandler->allStoreCartridgesCompleted('1.1.1','AAA')), 'not all store cartridges are completed');

    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','AAA','DE_EPAGES::CartridgeXZ');

    ok($PatchSystemInfoHandler->allStoreCartridgesCompleted('1.1.1','AAA'), 'now all store cartridges are completed');

    #store
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','AAA');

    #BBB
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','BBB', '1234567890');
    sleep(4);
    ok($PatchSystemInfoHandler->getProcessIDForStartedStore('BBB') eq '1234567890', 'correct processid');
    ok($PatchSystemInfoHandler->isStoreInProcess('BBB'), 'store is now in process with already used processid');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','BBB','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','BBB','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','BBB');
        
    is ($PatchSystemInfoHandler->getStoreDuration('BBB', '1.1.1'), 4, 'store duration correct');

    is ($PatchSystemInfoHandler->getSpendedStoreTimeMessage('BBB', '1.1.1', '1.1.1'), '4 s', 'spended store time message correct');
    
    #TestSTore1
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore1', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXA');
    sleep(4);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore1','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore1');

    #TestStore2
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore2', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXA');
    sleep(3);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore2','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore2');

    #TestStore3
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore3', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXA');
    sleep(5);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore3');

    #TestStore4
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore4', '1234567890');
    my $StartedStoreStartTime1 = $PatchSystemInfoHandler->getCurrentTime();
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore4','DE_EPAGES::CartridgeAZ');
    sleep(2);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore4','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore4');

    #TestStore5
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore5', '1234567890');
    my $StartedStoreStartTime2 = $PatchSystemInfoHandler->getCurrentTime();
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore5','DE_EPAGES::CartridgeAZ');
    sleep(3);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore5','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore5');

    #TestStore6
    $PatchSystemInfoHandler->registerStartedStore('1.1.1','TestStore6', '1234567890');
    my $StartedStoreStartTime3 = $PatchSystemInfoHandler->getCurrentTime();
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.1','TestStore6','DE_EPAGES::CartridgeAZ');
    sleep(4);
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.1','TestStore6','DE_EPAGES::CartridgeAZ');
    $PatchSystemInfoHandler->registerCompletedStore('1.1.1','TestStore6');

    #started store time
    is($PatchSystemInfoHandler->getStoreCartridgeStartTime('1.1.1', 'TestStore4', 'DE_EPAGES::CartridgeAZ'), $StartedStoreStartTime1, 'get store cartridge start time correct 1st');

    is($PatchSystemInfoHandler->getStoreCartridgeStartTime('1.1.1', 'TestStore5', 'DE_EPAGES::CartridgeAZ'), $StartedStoreStartTime2, 'get store cartridge start time correct 2nd');

    is($PatchSystemInfoHandler->getStoreCartridgeStartTime('1.1.1', 'TestStore6', 'DE_EPAGES::CartridgeAZ'), $StartedStoreStartTime3, 'get store cartridge start time correct 3rd');
    
    #store cartridge duration
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore1', 'DE_EPAGES::CartridgeXA'), 4, 'store cartridge duration correct 1st');
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXA'), 3, 'store cartridge duration correct 2nd');
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore3', 'DE_EPAGES::CartridgeXA'), 5, 'store cartridge duration correct 3rd');
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore4', 'DE_EPAGES::CartridgeAZ'), 2, 'store cartridge duration correct 4th');
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore5', 'DE_EPAGES::CartridgeAZ'), 3, 'store cartridge duration correct 5th');
    is($PatchSystemInfoHandler->getStoreCartridgeDuration('1.1.1', 'TestStore6', 'DE_EPAGES::CartridgeAZ'), 4, 'store cartridge duration correct 6th');

    is_deeply($PatchSystemInfoHandler->getStartedStores('1.1.1'), $aBusinessUnitStores, 'all started stores correct');

    ok($PatchSystemInfoHandler->allStoreStoresCompleted('1.1.1'), 'all stores are completed');

    ok($PatchSystemInfoHandler->isStoreCompleted('1.1.1','AAA'), 'store is now completed with patch');
    ok(not ($PatchSystemInfoHandler->isStoreInProcess('AAA')), 'store is no longer in process');
    ok(not ($PatchSystemInfoHandler->existsCurrentStoreVersion('AAA')), 'no store version exists yet');
    $PatchSystemInfoHandler->setCurrentStoreVersion('AAA','1.1.1');
    ok($PatchSystemInfoHandler->existsCurrentStoreVersion('AAA'), 'store version exists now');
    ok(($PatchSystemInfoHandler->getCurrentStoreVersion('AAA') eq '1.1.1'), 'store has correct version');

    #test cartridges not using standard version numbers
    $PatchSystemInfoHandler->registerStartedStore('1.1.2','TestStore3', '1234567890');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXY');
    $PatchSystemInfoHandler->registerStartedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXZ');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXA');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXB');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXG');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXH');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXK');
    $PatchSystemInfoHandler->registerCompletedStoreCartridge('1.1.2','TestStore3','DE_EPAGES::CartridgeXY');
      
    eval{
        $PatchSystemInfoHandler->registerCompletedStore('1.1.2','TestStore3');
    };
    ok( ExistsError(), 'correct error for complete store');

    $PatchSystemInfoHandler->setCartridgeUseStandardVersions('DE_EPAGES::CartridgeXZ', 0);

    eval{
        $PatchSystemInfoHandler->registerCompletedStore('1.1.2','TestStore3');
    };
    ok( not (ExistsError()), 'correct to complete store, ignore cartridges that dont using standard version numbers');

    #cartridges
    my $aCartridges = [
      'DE_EPAGES::CartridgeXA',
      'DE_EPAGES::CartridgeXB',
      'DE_EPAGES::CartridgeXG',
      'DE_EPAGES::CartridgeXH',
      'DE_EPAGES::CartridgeXK',
      'DE_EPAGES::CartridgeXY',
      'DE_EPAGES::CartridgeXZ'
    ];

    is_deeply( $PatchSystemInfoHandler->getStoreTypeCartridges('1.1.1','StoreType1'), $aCartridges, 'store type cartridges correct for store type \'StoreType1\'');
    is_deeply( $PatchSystemInfoHandler->getStoreCartridges('1.1.1','AAA'), $aCartridges, 'store cartridges correct for store \'AAA\'');
    is($PatchSystemInfoHandler->getStoreStoreType('AAA'), 'StoreType1', 'store type correct for store \'AAA\'');

    #test store password
    $PatchSystemInfoHandler->setStorePassword('AAA', 'ePages');
    $PatchSystemInfoHandler->setStorePassword('BBB', 'Jena');

    is ($PatchSystemInfoHandler->getStorePassword('AAA'), 'ePages', 'store password correct 1st');

    is ($PatchSystemInfoHandler->getStorePassword('BBB'), 'Jena', 'store password correct 2nd');
    
    #started patch function
    is(scalar @{$PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1','alphatesthost1')}, 0, 'no strted patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal', 'alphatesthost1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_basic', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1', 'alphatesthost1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal', 'alphatesthost1');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic', 'alphatesthost1');
    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1', 'alphatesthost1');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch', 'alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING', 'alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerStartedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch', 'alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aStartedPatchFunctions = [
        'API::Patch::PatchInstallHandlerStores::doPatch',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_basic',
        'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    is_deeply( $PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1', 'alphatesthost1'), $aStartedPatchFunctions, 'started patch functions correct for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerStartedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_1', 'testhost1');

    is_deeply( $PatchSystemInfoHandler->getStartedPatchFunctions('1.1.1', 'alphatesthost1'), $aStartedPatchFunctions, 'started patch functions correct for version \'1.1.1\' and host');

    #completed patch functions
    is(scalar @{$PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1','alphatesthost1')}, 0, 'no completed patch function for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal','alphatesthost1');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_basic','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1','alphatesthost1');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal','alphatesthost1');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic','alphatesthost1');
    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1','alphatesthost1');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::ANYTHING::doPatch','alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid handler throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_ANYTHING','alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid ground function name throws error');

    eval{$PatchSystemInfoHandler->registerCompletedPatchFunction('9.9.9', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch','alphatesthost1')};
    ok( ExistsError(), 'register patch funtion with invalid version throws error');

    my $aCompletedPatchFunctions = [
        'API::Patch::PatchInstallHandlerStores::doPatch',
        'API::Patch::PatchInstallHandlerStores::doPatch_1_1_1',
        'API::Patch::PatchInstallHandlerStores::doPatch_basic',
        'API::Patch::PatchInstallHandlerStores::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_basic',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerJQueryDictionaryConverter::doPatch_universal',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_1_1_1',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFolders::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_basic',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_checkPreCondition',
        'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_universal'
    ];

    is_deeply( $PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1','alphatesthost1'), $aCompletedPatchFunctions, 'completed patch functions correct for version \'1.1.1\'');

    $PatchSystemInfoHandler->registerCompletedPatchFunction('1.1.1', 'API::Patch::PatchPrepareHandlerVersionFoldersPatchCartridgeDir::doPatch_2_1_1', 'testhost1');

    is_deeply( $PatchSystemInfoHandler->getCompletedPatchFunctions('1.1.1', 'alphatesthost1'), $aCompletedPatchFunctions, 'completed patch functions correct for version \'1.1.1\' and host');

    #log level tests
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch') == 4, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('callSubHandlerWithVersionSources') == 3, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('toExecute') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_checkPreCondition') == 2, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_basic') == 1, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_universal') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('doPatch_2_1_3') == 0, 'correct loglevel');
    ok($PatchSystemInfoHandler->getGroundFunctionLogLevel('useNow') == 0, 'correct loglevel');

    #start time
    ok($StartTime eq $PatchSystemInfoHandler->getStartTime(), 'correct start time');

    #skip
    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 1);

    ok($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler'), "correct skip value");

    $PatchSystemInfoHandler->setHandlerSkip('API::Patch::PatchPrepareHandler', 0);

    ok(not ($PatchSystemInfoHandler->isHandlerToSkip('API::Patch::PatchPrepareHandler')), "correct skip value");

    ok(not ($PatchSystemInfoHandler->existsLanguage('en-US')), "language not exists");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US')), "language not exists in store for version");

    $PatchSystemInfoHandler->addLanguage('en');
        
    ok ($PatchSystemInfoHandler->isLanguageStandard('en-US'), '\'en-US\' is a standard language');

    ok (not ($PatchSystemInfoHandler->isLanguageStandard('la')), '\'la\' is not a standard language');
    
    ok($PatchSystemInfoHandler->existsLanguage('en'), "language exists");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en')), "language not exists in store for version");

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'en');

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','en')), "language not exists in store for version");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','en')), "language not exists in store for version");

    ok(not ($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US')), "language not exists in store for version");

    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en'), "language exists in store for version");

    my $aLanguages;

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'en-US');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','en-US'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'fr');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','fr'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'fi');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','fi'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore1', 'da');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore1','da'), "language exists in store for version");

    $aLanguages = ['da','en','en-US','fi','fr'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.1', 'TestStore1'), $aLanguages, 'correct languages');


    $PatchSystemInfoHandler->addLanguageToStore('1.1.2', 'TestStore1', 'es');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','es'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.2', 'TestStore1', 'no');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.2','TestStore1','no'), "language exists in store for version");

    $aLanguages = ['es', 'no'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.2', 'TestStore1'), $aLanguages, 'correct languages');

    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore2', 'it');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','it'), "language exists in store for version");
    $PatchSystemInfoHandler->addLanguageToStore('1.1.1', 'TestStore2', 'ru');
    ok($PatchSystemInfoHandler->existsStoreLanguageVersion('1.1.1','TestStore2','ru'), "language exists in store for version");

    $aLanguages = ['it', 'ru'];

    is_deeply( $PatchSystemInfoHandler->getStoreLanguages('1.1.1', 'TestStore2'), $aLanguages, 'correct languages');

    my $EpagesRealSharedPatchesDir = RealPath($ENV{EPAGES_SHARED}). '/Patches';
    $PatchSystemInfoHandler->setBasicAttribute('EpagesDataPackageDir', $EpagesRealSharedPatchesDir . '/DataPackages');
    $PatchSystemInfoHandler->setBasicAttribute('EpagesSharedPatchesDir', $EpagesRealSharedPatchesDir) unless $PatchSystemInfoHandler->existsBasicAttribute('EpagesSharedPatchesDir');
    ok(($PatchSystemInfoHandler->getCartridgesDataPackageFile('1.1.1') =~ /epages-1\.1\.1-patch-data\.tar\.gz/) , "cartridges data package correct for version '1.1.1'");

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesDataPackageExclusiveURLs', '/opt/eproot/Shared/RPMS');
    ok(($PatchSystemInfoHandler->isLocalCartridgesDataPackageRepo()), 'correct check local exclusive repo 1st');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesDataPackageExclusiveURLs',
        "http://ep-server/pub/epages/build/repo/epages
         ftp://epages-software.de/repo/epages
         http://epages.com/repo/epages"
    );
    ok(not ($PatchSystemInfoHandler->isLocalCartridgesDataPackageRepo()), 'correct check local exclusive repo 2nd');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesDataPackageExclusiveURLs',
        "http://ep-server/pub/epages/build/repo/epages
         ftp://epages-software.de/repo/epages
         http://epages.com/repo/epages
         /opt/eproot/Shared/RPMS"
    );

    eval{$PatchSystemInfoHandler->isLocalCartridgesDataPackageRepo()};
    ok( ExistsError(), 'register mixed case repos');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesDataPackageExclusiveURLs',
        "/opt/XXX/Shared/RPMS
        D:\epages/Shared/RPMS
        /opt/eproot/Shared/RPMS"
    );

    eval{$PatchSystemInfoHandler->isLocalCartridgesDataPackageRepo()};
    ok( ExistsError(), 'register wrong case repos');

    $PatchSystemInfoHandler->setBasicAttribute('PatchCartridgesDataPackageExclusiveURLs',
        "

        D:\epages/Shared/RPMS

        "
    );

    ok(($PatchSystemInfoHandler->isClusterPatchVersion('6.17.9.3')), 'correct cluster version 1st');

    ok(($PatchSystemInfoHandler->isClusterPatchVersion('6.17.9.3.1')), 'correct cluster version 2nd');

    ok( not ($PatchSystemInfoHandler->isClusterPatchVersion('6.17.9')), 'correct cluster version 3rd');

    ok(($PatchSystemInfoHandler->isMainPatchVersion('6.17.9')), 'correct main patch version 1st');

    ok( not ($PatchSystemInfoHandler->isMainPatchVersion('6.17.9.3')), 'correct cluster version 2nd');

    ok( not ($PatchSystemInfoHandler->isMainPatchVersion('6.17.9.3.1')), 'correct cluster version 3rd');

    ok( not ($PatchSystemInfoHandler->isMainPatchVersion('6.17')), 'correct cluster version 4th');

    #test duration computation
    $PatchSystemInfoHandler->registerStartedCartridgeFunction('1.2.3', 'TestStore1', 'DE_EPAGES::CartridgeXY','patch');
    sleep(7);

    ok($PatchSystemInfoHandler->isCartridgeFunctionStarted('1.2.3', 'TestStore1', 'DE_EPAGES::CartridgeXY','patch'), '\'patch\' function is started for \'DE_EPAGES::CartridgeXY\'');
    
    $PatchSystemInfoHandler->registerCompletedCartridgeFunction('1.2.3', 'TestStore1', 'DE_EPAGES::CartridgeXY','patch');

    ok($PatchSystemInfoHandler->isCartridgeFunctionCompleted('1.2.3', 'TestStore1', 'DE_EPAGES::CartridgeXY','patch'), '\'patch\' function is completed for \'DE_EPAGES::CartridgeXY\'');

    $PatchSystemInfoHandler->registerStartedCartridgeFunction('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','install');
    sleep(14);

    ok($PatchSystemInfoHandler->isCartridgeFunctionStarted('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','install'), '\'install\' function is started for \'DE_EPAGES::CartridgeXB\'');

    $PatchSystemInfoHandler->registerCompletedCartridgeFunction('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','install');

    ok ($PatchSystemInfoHandler->getCartridgeFunctionDuration('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB', 'install') == 14, 'cartridge function duration correct');
    
    ok($PatchSystemInfoHandler->isCartridgeFunctionCompleted('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','install'), '\'install\' function is completed for \'DE_EPAGES::CartridgeXB\'');
    
    ok(not ($PatchSystemInfoHandler->isCartridgeFunctionStarted('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','patchCartridgeDirFinalPart')), '\'patchCartridgeDirFinalPart\' function is not started for \'DE_EPAGES::CartridgeXB\'');
    
    ok(not ($PatchSystemInfoHandler->isCartridgeFunctionCompleted('2.1.1', 'TestStore2', 'DE_EPAGES::CartridgeXB','patchCartridgeDirFinalPart')), '\'patchCartridgeDirFinalPart\' function is not completed for \'DE_EPAGES::CartridgeXB\'');

    my $EndTimeTest = $PatchSystemInfoHandler->getCartridgeFunctionCompleteTime('1.2.3','TestStore1','DE_EPAGES::CartridgeXY','patch');
    my $StartTimeTest = $PatchSystemInfoHandler->getCartridgeFunctionStartTime('1.2.3','TestStore1','DE_EPAGES::CartridgeXY','patch');

    ok(($PatchSystemInfoHandler->getDurationInSeconds($StartTimeTest, $EndTimeTest) == 7), 'correct duration in seconds 1st');

    $EndTimeTest = $PatchSystemInfoHandler->getCartridgeFunctionCompleteTime('2.1.1','TestStore2','DE_EPAGES::CartridgeXB','install');
    $StartTimeTest = $PatchSystemInfoHandler->getCartridgeFunctionStartTime('2.1.1','TestStore2','DE_EPAGES::CartridgeXB','install');
    ok(($PatchSystemInfoHandler->getDurationInSeconds($StartTimeTest, $EndTimeTest) == 14), 'correct duration in seconds 2nd');

    #check test coverage for PatchSystemInfoHandler
    my $TestCoverage = $PatchSystemInfoHandler->getTestCoverage();
    ok ($TestCoverage >= 80, "TestCoverage for PatchSystemInfoHandler is '80%' or higher ( Coverage: '$TestCoverage' )");
    
    Cleanup($CurrentDir);
}

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [flags]

flags:
    -help          show the command line options

Example:
    perl $0
END_USAGE
    exit 2;
}

RunScript( Message => 'long', Sub => \&Main);

1;
